@@ -6,15 +6,20 @@ import logging
 import time
 import threading
 
+import sqlalchemy as sa
 from falcon import testing, errors
 
 from nostr_relay.config import Config
-from nostr_relay.web import create_app, get_storage
-from nostr_relay.event import Event, PrivateKey
+from nostr_relay.web import create_app
+from nostr_relay.storage import get_storage
 from nostr_relay.auth import Authenticator
+from nostr_relay.event import Event, PrivateKey
+from nostr_relay.errors import StorageError
+
 
 from nostr_relay import __version__
 
+from . import BaseTestsWithStorage
 
 PK1 = 'f6d7c79924aa815d0d408bc28c1a23af208209476c1b7691df96f7d7b72a2753'
 PK2 = '8f50290eaa19f3cefc831270f3c2b5ddd3f26d11b0b72bc957067d6811bc618d'
@@ -33,28 +38,155 @@ REPLACE_EVENTS = [
 DELEGATION_EVENT = { "id": "a080fd288b60ac2225ff2e2d815291bd730911e583e177302cc949a15dc2b2dc",  "pubkey": "62903b1ff41559daf9ee98ef1ae67cc52f301bb5ce26d14baba3052f649c3f49",  "created_at": 1660896109,  "kind": 1, "tags":[["delegation","86f0689bd48dcd19c67a19d994f938ee34f251d8c39976290955ff585f2db42e","kind=1&created_at>1640995200","c33c88ba78ec3c760e49db591ac5f7b129e3887c8af7729795e85a0588007e5ac89b46549232d8f918eefd73e726cb450135314bfda419c030d0b6affe401ec1"]],  "content": "Hello world",  "sig": "cd4a3cd20dc61dcbc98324de561a07fd23b3d9702115920c0814b5fb822cc5b7c5bcdaf3fa326d24ed50c5b9c8214d66c75bae34e3a84c25e4d122afccb66eb6"}
 
 
-class BaseTests(unittest.IsolatedAsyncioTestCase):
-    @classmethod
-    def setUpClass(cls):
-        Config.load(os.path.join(os.path.dirname(__file__), './test_config.yaml'))
 
-    def setUp(self):
-        pass
-        # logging.disable(logging.CRITICAL)
+class DBTests(BaseTestsWithStorage):
+    async def test_add_bad_event(self):
+        with self.assertRaises(StorageError) as e:
+            await self.storage.add_event([1,2,3])
+        assert e.exception.args[0] == 'invalid: Bad JSON'
+
+        with self.assertRaises(StorageError) as e:
+            evt = self.make_event(PK1, pubkey=PK2)
+            await self.storage.add_event(evt)
+        assert e.exception.args[0] == 'invalid: Bad signature'
+
+        with self.assertRaises(StorageError) as e:
+            evt = self.make_event(PK1, created_at=123)
+            await self.storage.add_event(evt)
+        assert e.exception.args[0] == 'invalid: 123 is too old'
+
+        with self.assertRaises(StorageError) as e:
+            t = int(time.time() + 86400)
+            evt = self.make_event(PK1, created_at=t)
+            await self.storage.add_event(evt)
+        assert e.exception.args[0] == f'invalid: {t} is in the future'
+
+        with self.assertRaises(StorageError) as e:
+            evt = self.make_event(PK1, content='x' * 64000)
+            await self.storage.add_event(evt)
+        assert e.exception.args[0] == f'invalid: 280 characters should be enough for anybody'
+
+    async def test_add_event(self):
+        event, changed = await self.storage.add_event(EVENTS[0])
+        assert event.id == EVENTS[0]['id']
+        assert changed
+
+        event, changed = await self.storage.add_event(EVENTS[0])
+        assert event.id == EVENTS[0]['id']
+        assert not changed
+
+    async def test_replaceable_event(self):
+        evt1 = self.make_event(PK1, kind=11111, content='event 1', tags=[['e', '1234'], ['foo', 'bar']])
+        event, changed = await self.storage.add_event(evt1)
+        assert event.id == evt1['id']
+
+        evt2 = self.make_event(PK1, kind=11111, content='event 2')
+        event, changed = await self.storage.add_event(evt2)
+        assert event.id == evt2['id']
+
+        # first event should be gone
+        evt = await self.storage.get_event(evt1['id'])
+        assert evt is None
+
+        # an event from a different pubkey won't replace event 2
+        evt3 = self.make_event(PK2, kind=11111, content='event 3')
+        event, changed = await self.storage.add_event(evt2)
+        assert event.id == evt2['id']
+        evt = await self.storage.get_event(evt2['id'])
+        assert evt['content'] == 'event 2'
+
+    async def test_expiration_tag(self):
+        expiring_event = {"id": "075040fbf395db975fccf36948908474994f70c871be0d1755459851438577d1", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 1, "tags": [["expiration", 1672329427]], "content": "this will expire", "sig": "77509c595b5e4e86a74dc1fa0b4a484a8bca6acc89e2e32b9794c9e0c59a0313521295e8c4b6b844be5ce83bab2cb7230bc0d9761fbf3845a2cd04dab2d70cda"}
+        distant_future_event = {"id": "eb41e9b29e08aa73bb73ebe6daa73ce59b484cfe62b95449d0bcd551186ee61c", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 1, "tags": [["expiration", 1987990393]], "content": "this will expire in a long time", "sig": "046d4ad7eb8b7b64528cfcde2f4171d8ad375467447a134dbd14b764973a237f977c3f8a5d3ba6cfcc5ed07d0fa68b222a18c81c2c9c115bdd55734c7006877b"}
+
+        await self.storage.add_event(expiring_event)
+        await self.storage.add_event(distant_future_event)
+        assert (await self.storage.get_event(expiring_event["id"]))['id'] == expiring_event['id']
+        assert (await self.storage.get_event(distant_future_event["id"]))['id'] == distant_future_event['id']
+
+        # stop and restart garbage collector to have it run immediately
+        from nostr_relay.storage.db import start_garbage_collector
+        # self.storage.garbage_collector.stop()
+        self.storage.garbage_collector = start_garbage_collector(self.storage.db, {'collect_interval': 2})
+
+        await asyncio.sleep(2.5)
+        assert (await self.storage.get_event(expiring_event["id"])) is None
+        self.storage.garbage_collector.cancel()
+        assert (await self.storage.get_event(distant_future_event["id"]))['id'] == distant_future_event['id']
+
+    async def test_ephemeral_event(self):
+        from nostr_relay.storage.db import start_garbage_collector
+        self.storage.garbage_collector_task = start_garbage_collector(self.storage.db, {'collect_interval': 2})
+        ephemeral_event = {"id": "2696df86ce47142b7d272408e222b7a9fc4b2cc3a428bf2debf5d730ae2f42c7", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 22222, "tags": [], "content": "ephemeral", "sig": "66f8a055bb3c3fc3fe0ca0ead4d5558d69627dc4f40c7320228d9e4c266509f6ac8a2ff085abbd1a9d3b0c733529bf3fcd87d43f731990467181ed1995aad5bc"}
+
+        assert (await self.storage.add_event(ephemeral_event))[1]
+        assert (await self.storage.get_event(ephemeral_event['id']))['id'] == '2696df86ce47142b7d272408e222b7a9fc4b2cc3a428bf2debf5d730ae2f42c7'
+        await asyncio.sleep(2.2)
+        assert (await self.storage.get_event(ephemeral_event['id'])) is None
+
+    async def test_delete_event(self):
+        evt1 = self.make_event(PK1, kind=1, content='delete me')
+        event, changed = await self.storage.add_event(evt1)
+
+        delete_event = self.make_event(PK2, kind=5)
+        await self.storage.add_event(delete_event)
+        assert (await self.storage.get_event(evt1['id'])) is not None
+
+        delete_event = self.make_event(PK2, kind=5, tags=[['e', evt1['id']]])
+        await self.storage.add_event(delete_event)
+        assert (await self.storage.get_event(evt1['id'])) is not None
+
+        delete_event = self.make_event(PK1, kind=5, tags=[['e', evt1['id']]])
+        await self.storage.add_event(delete_event)
+        assert (await self.storage.get_event(evt1['id'])) is None
+
+    async def test_delegation_event(self):
+        event, changed = await self.storage.add_event(DELEGATION_EVENT)
+        assert changed
+        import json
+        async for event in self.storage.run_single_query([{"authors": ["86f0689bd48dcd19c67a19d994f938ee34f251d8c39976290955ff585f2db42e"]}]):
+            assert json.loads(event)['id'] == DELEGATION_EVENT['id']
+
+    async def test_get_stats(self):
+        for evt in EVENTS:
+            await self.storage.add_event(evt)
+
+        queue = asyncio.Queue()
+        sub = await self.storage.subscribe('test', 'test', [{'kinds': [1]}], queue)
+        stats = await self.storage.get_stats()
+        assert stats['total'] == 3
+        assert stats['active_subscriptions'] == 1
+        await self.storage.unsubscribe('test')
+
+    async def test_set_idp_identifier(self):
+        await self.storage.set_identified_pubkey('test@localhost', '5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d', relays='ws://localhost:6969')
+
+        data = await self.storage.get_identified_pubkey('test@localhost')
+        assert data == {
+            'names': {
+                'test': '5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d'
+            },
+            'relays': {
+                '5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d': 'ws://localhost:6969'
+            }
+        }
+
+        data = await self.storage.get_identified_pubkey('', domain='foo.com')
+        assert data == {'names': {}, 'relays': {}}
+
+        with self.assertRaises(StorageError):
+            await self.storage.set_identified_pubkey('test@localhost', 'asd')
+
+        await self.storage.set_identified_pubkey('test@localhost', '')
+        data = await self.storage.get_identified_pubkey('test@localhost')
+        assert data['names'] == {}
 
 
+class APITests(BaseTestsWithStorage):
     async def asyncSetUp(self):
-        self.storage = get_storage(reload=True)
-
+        await super().asyncSetUp()
         self.conductor = testing.ASGIConductor(create_app(storage=self.storage))
 
-        await self.storage.setup_db()
-
-    async def asyncTearDown(self):
-        await self.storage.db.execute("DELETE from event")
-        await self.storage.db.commit()
-        await self.storage.close()
-
     async def send_event(self, ws, event, get_response=False):
         await ws.send_json(["EVENT", event])
         if get_response:
@@ -76,11 +208,11 @@ class BaseTests(unittest.IsolatedAsyncioTestCase):
         return data
 
 
-class MainTests(BaseTests):
+class MainTests(APITests):
     async def test_get_event(self):
         async with self.conductor.simulate_ws('/') as ws:
             await self.send_event(ws, EVENTS[0], True)
-        result = await self.conductor.simulate_get(f'/event/{EVENTS[0]["id"]}')
+        result = await self.conductor.simulate_get(f'/e/{EVENTS[0]["id"]}')
         data = result.json
         assert data == EVENTS[0]
 
@@ -102,7 +234,7 @@ class MainTests(BaseTests):
                 20,
                 26,
                 40,
-                42,
+                # 42,
             ],
             'version': __version__,
         }
@@ -152,13 +284,6 @@ class MainTests(BaseTests):
             data = await self.send_event(ws, EVENTS[0], True)
             assert data == ['OK', '0d7721e1ee4a343f623cfb86374cc2d4688784bd264b5bc26079843169f28d88', False, 'duplicate: exists']
 
-            # check for existence
-            data = await self.get_event(ws, EVENTS[0]["id"])
-
-            for event in EVENTS:
-                await self.send_event(ws, event, True)
-
-
     async def test_req(self):
         async with self.conductor.simulate_ws('/') as ws:
             for event in EVENTS:
@@ -207,6 +332,10 @@ class MainTests(BaseTests):
             data = await ws.receive_json()
             assert data == ['EOSE', 'test']
 
+            await ws.send_json(["REQ", "missing", {"kinds": [1000]}])
+            data = await ws.receive_json()
+            assert data == ['EOSE', 'missing']
+
             # now add a new event
             data = await self.send_event(ws, EVENTS[2], True)
             if data[0] == 'OK':
@@ -244,81 +373,6 @@ class MainTests(BaseTests):
             data = await ws.receive_json()
             assert data == ['EOSE', 'junk']
 
-    async def test_replace_events(self):
-        async with self.conductor.simulate_ws('/') as ws:
-            await self.send_event(ws, REPLACE_EVENTS[0], True)
-            await self.send_event(ws, REPLACE_EVENTS[1], True)
-            await ws.send_json(["REQ", "test", {"kinds": [11111]}])
-            data = await ws.receive_json()
-            assert data == ['EVENT', 'test', REPLACE_EVENTS[1]]
-            data = await ws.receive_json()
-            assert data == ['EOSE', 'test']
-
-    async def test_delegation_event(self):
-        async with self.conductor.simulate_ws('/') as ws:
-            await self.send_event(ws, DELEGATION_EVENT, True)
-            await ws.send_json(["REQ", "delegation", {"authors": ["86f0689bd48dcd19c67a19d994f938ee34f251d8c39976290955ff585f2db42e"]}])
-            data = await ws.receive_json()
-            assert data == ["EVENT", "delegation", DELEGATION_EVENT]
-
-    async def test_delete_event(self):
-        async with self.conductor.simulate_ws('/') as ws:
-            await self.send_event(ws, EVENTS[1], True)
-            await self.send_event(ws, EVENTS[2], True)
-
-            # send valid delete event
-            response = await self.send_event(ws, {"id": "6b91c6821f4b480c493b4a07e736675ed078fdcdf71a3918fe59e3d1ef2da907", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 5, "tags": [["e", "46981a47c7e28f720a2609a5872c62b6e8b9ae612db3f8320b68be446ad77e11"]], "content": "delete mistake", "sig": "f6a41a55c4b1bf964db2d67a14b1680c6ba39f7216fc45b195d760eb6b9308c73694dd7cc61283562409328daedea3a733e8e692ded46bb3e61e2195ab0ce655"}, True)
-            assert response == ["OK", "6b91c6821f4b480c493b4a07e736675ed078fdcdf71a3918fe59e3d1ef2da907", True, '']
-
-            # event should be deleted
-            await ws.send_json(["REQ", "deleted", {"ids": ["46981a47c7e28f720a2609a5872c62b6e8b9ae612db3f8320b68be446ad77e11"]}])
-            data = await ws.receive_json()
-            assert data == ['EOSE', 'deleted']
-
-            # send invalid delete event
-            response = await self.send_event(ws, {"id": "61b5e7adaefaefc600ae838806667d87ca9aef19993707190f0fffa2dbee71b0", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 5, "tags": [["e", "ea54cf0e912b3fc482ee842195c8db670df6d958b2ae86e039b7c56690d81ef9"]], "content": "bad deletion", "sig": "6a274fa7c18fe34fcc29aa4690286941c288ef57d1ff083645f96549276ea7c4655ddc03a05fcc7167483247f4aadde71d89794421f24da7f7958e065e44744c"}, True)
-            assert response == ["OK", "61b5e7adaefaefc600ae838806667d87ca9aef19993707190f0fffa2dbee71b0", True, '']
-
-            # event will not be deleted
-            await ws.send_json(["REQ", "deleted", {"ids": ["ea54cf0e912b3fc482ee842195c8db670df6d958b2ae86e039b7c56690d81ef9"]}])
-            data = await ws.receive_json()
-            assert data == ['EVENT', 'deleted', EVENTS[2]]
-
-    async def test_ephemeral_event(self):
-        from nostr_relay.db import start_garbage_collector
-        # self.storage.garbage_collector.stop()
-        self.storage.garbage_collector_task = start_garbage_collector(self.storage.db, {'collect_interval': 3})
-        ephemeral_event = {"id": "2696df86ce47142b7d272408e222b7a9fc4b2cc3a428bf2debf5d730ae2f42c7", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 22222, "tags": [], "content": "ephemeral", "sig": "66f8a055bb3c3fc3fe0ca0ead4d5558d69627dc4f40c7320228d9e4c266509f6ac8a2ff085abbd1a9d3b0c733529bf3fcd87d43f731990467181ed1995aad5bc"}
-
-        async with self.conductor.simulate_ws("/") as ws:
-            data = await self.send_event(ws, ephemeral_event, True)
-            assert data[2] == True
-            data = await self.get_event(ws, ephemeral_event["id"], exists=True)
-            await asyncio.sleep(3.5)
-            data = await self.get_event(ws, ephemeral_event["id"], exists=False)
-        # self.storage.garbage_collector.stop()
-
-    async def test_expiration_tag(self):
-
-        async with self.conductor.simulate_ws("/") as ws:
-            expiring_event = {"id": "075040fbf395db975fccf36948908474994f70c871be0d1755459851438577d1", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 1, "tags": [["expiration", 1672329427]], "content": "this will expire", "sig": "77509c595b5e4e86a74dc1fa0b4a484a8bca6acc89e2e32b9794c9e0c59a0313521295e8c4b6b844be5ce83bab2cb7230bc0d9761fbf3845a2cd04dab2d70cda"}
-            distant_future_event = {"id": "eb41e9b29e08aa73bb73ebe6daa73ce59b484cfe62b95449d0bcd551186ee61c", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 1, "tags": [["expiration", 1987990393]], "content": "this will expire in a long time", "sig": "046d4ad7eb8b7b64528cfcde2f4171d8ad375467447a134dbd14b764973a237f977c3f8a5d3ba6cfcc5ed07d0fa68b222a18c81c2c9c115bdd55734c7006877b"}
-
-            await self.send_event(ws, expiring_event, True)
-            await self.send_event(ws, distant_future_event, True)
-            await self.get_event(ws, expiring_event["id"], exists=True)
-            await self.get_event(ws, distant_future_event["id"], exists=True)
-
-            # stop and restart garbage collector to have it run immediately
-            from nostr_relay.db import start_garbage_collector
-            # self.storage.garbage_collector.stop()
-            self.storage.garbage_collector = start_garbage_collector(self.storage.db, {'collect_interval': 3})
-
-            await asyncio.sleep(3.5)
-            await self.get_event(ws, expiring_event["id"], exists=False)
-            self.storage.garbage_collector.cancel()
-            await self.get_event(ws, distant_future_event["id"], exists=True)
-
     async def test_tag_search(self):
         async with self.conductor.simulate_ws("/") as ws:
             tag_event = {"id": "0b5d4c896b9f8b74feb0a63f8c50536b094ae948049d4c3ab967afc98250cee3", "pubkey": "5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d", "created_at": 1672325827, "kind": 1, "tags": [["t", "test"], ["t", "foo"], ["h", "home"]], "content": "got tags", "sig": "fbbd5fcf0e385d39d6120735f49c52ccc9f278670d7ce0c4ff774ff454fd810c4d18c005db681bbb5754d19351b922570f18d3fa8fae697c0480ebfccbed06e7"}
@@ -355,10 +409,10 @@ class MainTests(BaseTests):
             assert data[3] == 'rate-limited: slow down'
 
 
-class AuthTests(BaseTests):
+class AuthTests(APITests):
     async def asyncSetUp(self):
+        Config.authentication = {"enabled": True, "actions": {"save": "w", "query": "r"}}
         await super().asyncSetUp()
-        self.storage.authenticator = Authenticator(self.storage, {"enabled": True, "actions": {"save": "w", "query": "r"}})
         self.readonly = ('f6d7c79924aa815d0d408bc28c1a23af208209476c1b7691df96f7d7b72a2753', '5faaae4973c6ed517e7ed6c3921b9842ddbc2fc5a5bc08793d2e736996f6394d', 'r')
         self.writeonly = ('e93505f081570221255f05341f4fbaeaf682d59d2e2472d7dd02d566f6372178', '647bf6e686fde33dfbfd4f3d987bc13b1c202f952b70dc3539d8d85172b40561', 'w')
         self.readwrite = ('9679a595bdb5fe4330a263c96201eac204dfe24b2e2dc36ac12698faf9275130', '1bd80e4430c40a6fa9f59582124b5a6fbc1815e26455801e6b1edf113554de03', 'rw')
@@ -368,31 +422,42 @@ class AuthTests(BaseTests):
             await self.storage.authenticator.set_roles(pubkey, role)
 
     async def asyncTearDown(self):
-        await self.storage.db.execute("DELETE from auth")
+        from nostr_relay.auth import AuthTable
+        async with self.storage.db.begin() as conn:
+            await conn.execute(AuthTable.delete())
         await super().asyncTearDown()
 
-    def make_auth_event(self, privkey, pubkey, kind=22242, created_at=None, content='ws://localhost:6969', **kwargs):
-        auth_event = Event(kind=kind, pubkey=pubkey, created_at=created_at or time.time(), content=content, **kwargs)
-        auth_event.sign(privkey)
-        return auth_event.to_json_object()
+    async def get_challenge(self, ws):
+        data = await ws.receive_json()
+        assert data[0] == 'AUTH'
+        return data[1]
+
+    def make_auth_event(self, privkey, pubkey, created_at=None, challenge='', relay='ws://localhost:6969', **kwargs):
+        tags = [
+            ['relay', relay],
+            ['challenge', challenge]
+        ]
+        return self.make_event(privkey, pubkey=pubkey, kind=22242, created_at=created_at, tags=tags, **kwargs)
 
     async def test_auth_message(self):
         """
         Test AUTH message type
         """
         async with self.conductor.simulate_ws("/") as ws:
+            # server sends AUTH message challenge
+            challenge = await self.get_challenge(ws)
+
             # send one with a bad signature
-            await ws.send_json(["AUTH", self.make_auth_event(self.unknown[0], self.readonly[1])])
+            await ws.send_json(["AUTH", self.make_auth_event(self.unknown[0], self.readonly[1], challenge=challenge)])
             data = await ws.receive_json()
             assert data == ['NOTICE', 'invalid: Bad signature']
 
-            await ws.send_json(["AUTH", self.make_auth_event(self.readonly[0], self.readonly[1], created_at=time.time() - 3600)])
+            await ws.send_json(["AUTH", self.make_auth_event(self.readonly[0], self.readonly[1], challenge=challenge, created_at=time.time() - 3600)])
             data = await ws.receive_json()
             assert data == ['NOTICE', 'invalid: Too old']
 
             # correct auth sends no response
-            await ws.send_json(["AUTH", self.make_auth_event(self.readonly[0], self.readonly[1])])
-
+            await ws.send_json(["AUTH", self.make_auth_event(self.readonly[0], self.readonly[1], challenge=challenge)])
 
     async def test_role_permissions(self):
         """
@@ -400,11 +465,13 @@ class AuthTests(BaseTests):
         """
         # test write permission
         async with self.conductor.simulate_ws("/") as ws:
+            challenge = await self.get_challenge(ws)
+
             response = await self.send_event(ws, EVENTS[1], True)
             assert response[2] == False
-            assert response[3] == 'rejected: permission denied'
+            assert response[3] == 'restricted: permission denied'
 
-            await ws.send_json(["AUTH", self.make_auth_event(self.writeonly[0], self.writeonly[1])])
+            await ws.send_json(["AUTH", self.make_auth_event(self.writeonly[0], self.writeonly[1], challenge=challenge)])
 
             response = await self.send_event(ws, EVENTS[1], True)
             assert response[2] == True
@@ -413,23 +480,25 @@ class AuthTests(BaseTests):
             # role is write only, so read will fail
             await ws.send_json(["REQ", "read", {"ids": [EVENTS[1]["id"]]}])
             data = await ws.receive_json()
-            assert data == ['NOTICE', 'rejected: permission denied']
+            assert data == ['NOTICE', 'restricted: permission denied']
 
         # test read permission
         async with self.conductor.simulate_ws("/") as ws:
+            challenge = await self.get_challenge(ws)
+
             response = await self.send_event(ws, EVENTS[2], True)
             assert response[2] == False
-            assert response[3] == 'rejected: permission denied'
+            assert response[3] == 'restricted: permission denied'
 
             await ws.send_json(["REQ", "read", {"ids": [EVENTS[1]["id"]]}])
             data = await ws.receive_json()
-            assert data == ['NOTICE', 'rejected: permission denied']
+            assert data == ['NOTICE', 'restricted: permission denied']
 
             # authenticate as read only role
-            await ws.send_json(["AUTH", self.make_auth_event(self.readonly[0], self.readonly[1])])
+            await ws.send_json(["AUTH", self.make_auth_event(self.readonly[0], self.readonly[1], challenge=challenge)])
             response = await self.send_event(ws, EVENTS[2], True)
             assert response[2] == False
-            assert response[3] == 'rejected: permission denied'
+            assert response[3] == 'restricted: permission denied'
 
             await ws.send_json(["REQ", "read", {"ids": [EVENTS[1]["id"]]}])
             data = await ws.receive_json()
@@ -439,16 +508,17 @@ class AuthTests(BaseTests):
 
         # test unknown role
         async with self.conductor.simulate_ws("/") as ws:
-            await ws.send_json(["AUTH", self.make_auth_event(self.unknown[0], self.unknown[1])])
+            challenge = await self.get_challenge(ws)
+            await ws.send_json(["AUTH", self.make_auth_event(self.unknown[0], self.unknown[1], challenge=challenge)])
 
             # authenticated, but can't do anything
             response = await self.send_event(ws, EVENTS[2], True)
             assert response[2] == False
-            assert response[3] == 'rejected: permission denied'
+            assert response[3] == 'restricted: permission denied'
 
             await ws.send_json(["REQ", "read", {"ids": [EVENTS[1]["id"]]}])
             data = await ws.receive_json()
-            assert data == ['NOTICE', 'rejected: permission denied']
+            assert data == ['NOTICE', 'restricted: permission denied']
 
 
 if __name__ == "__main__":
